#include "sessiondbusregister.h"
#include <QMap>
#include <QDateTime>

#include "globalsize.h"

#define TIMER_TIMEOUT   (2*1000)

extern qint64 powerChangeTime;
extern bool resetFlag;
extern bool sleepFlag;

static bool isScan = false;
static SessionDbusRegister * static_sess = nullptr;
static QMap<QString, DelayStruct *> static_delays;

//检测bluez重启参数
//bluez重启(停止然后启动)后，关闭适配器蓝牙，移除蓝牙适配器，新增蓝牙适配器
static int static_bluezreboot_power = -1;

//仅适合单线程模式
int cancelEmitDevStatus(QString addr)
{
    DelayStruct * p = static_delays.value(addr, nullptr);
    if(nullptr != p)
    {
        p->close();
        //从qmap删除
        static_delays.remove(addr);
    }
    return 0;
}

DelayStruct::DelayStruct(BluezQt::DevicePtr dev, QObject *parent /*= nullptr*/):QObject(parent)
{
    m_dev = dev;
    m_TimerID = this->startTimer(TIMER_TIMEOUT);
}

DelayStruct::~DelayStruct()
{
    qDebug() << Q_FUNC_INFO << __LINE__;
    m_dev = nullptr;
}

void  DelayStruct::close(void)
{
    qDebug() << Q_FUNC_INFO << __LINE__;
    this->killTimer(m_TimerID);
    this->deleteLater();
}

void DelayStruct::timerEvent( QTimerEvent *event)
{
    qDebug() << Q_FUNC_INFO << __LINE__;
    if(nullptr != m_dev && nullptr != static_sess)
    {
        qDebug() << Q_FUNC_INFO << __LINE__ << m_dev->address() << m_dev->isConnected();
        emit static_sess->devConnectStatusSignal(m_dev->address(), m_dev->isConnected());
        if (m_dev->isConnected() && m_dev->isPaired()) {
            Config::writeKeyFile(m_dev->address(),m_dev->name(),m_dev->type());
        }
    }
    //从qmap删除
    if(nullptr != m_dev)
    {
        static_delays.remove(m_dev->address());
    }

    this->close();
}


static void init_static_sess(SessionDbusRegister * p)
{
    static_sess = p;
}

void addDelay(BluezQt::DevicePtr dev)
{
    if(nullptr != dev)
    {
        QString key = dev->address();
        //未找到
        if(static_delays.find(key) == static_delays.end())
        {
            DelayStruct * p = new DelayStruct(dev);
            static_delays[key] = p;
        }
    }
}

static bool existDelay(QString key)
{
    return (static_delays.find(key) != static_delays.end());
}

////////////////////////////////////////////SessionDbusRegister////////////////////////////////////
////////////////////////////////////////////SessionDbusRegister////////////////////////////////////


static inline bool isAvailableAddress(QString addr) {
    if (addr.isEmpty() || addr.isNull())
        return false;

    return true;
}

SessionDbusRegister::SessionDbusRegister(QObject *parent) : QObject(parent)
{
    qInfo() << Q_FUNC_INFO ;
    init_static_sess(this);
    if (InterfaceAlreadyExists()) {
        QDBusConnection sessionDbus = QDBusConnection::sessionBus();
        if (!sessionDbus.registerService("com.ukui.bluetooth")) {
            qCritical() << "QDbus register service failed reason:" << sessionDbus.lastError();
        }

        if (!sessionDbus.registerObject("/com/ukui/bluetooth", "com.ukui.bluetooth", this, QDBusConnection::ExportAllSlots|QDBusConnection::ExportAllSignals)){
            qCritical() << "QDbus register object failed reason:" << sessionDbus.lastError();
        }
    }
}

SessionDbusRegister::~SessionDbusRegister()
{

}

bool SessionDbusRegister::InterfaceAlreadyExists()
{
    QDBusConnection conn = QDBusConnection::sessionBus();
    if (!conn.isConnected())
        return 0;

    QDBusReply<QString> reply = conn.interface()->call("GetNameOwner", "com.ukui.bluetooth");
    return reply.value() == "";
}

void SessionDbusRegister::AudioDevConnectFunc(BluezQt::DevicePtr dev)
{
    qDebug () << Q_FUNC_INFO  << "START" << __LINE__;

    if (dev)
    {
//        if (Daemon::_DefualtAdapter.data()->isDiscovering()) {
//            ::isScan = true;
//            BluezQt::PendingCall * stop_call = Daemon::_DefualtAdapter.data()->stopDiscovery();
//            connect(stop_call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn)
//            {
//                qDebug() << Q_FUNC_INFO << "stopDiscovery:" << callReturn->error() << callReturn->errorText() << __LINE__;
//            });
//            //qDebug() << Q_FUNC_INFO << "stopDiscovery:111111111111111111" << __LINE__;
//            stop_call->waitForFinished();
//            //qDebug() << Q_FUNC_INFO << "stopDiscovery:222222222222222222" << __LINE__;

//        }

        if (dev.data()->type() == BluezQt::Device::AudioVideo ||
            dev.data()->type() == BluezQt::Device::Headphones ||
            dev.data()->type() == BluezQt::Device::Headset    ) {

            BluezQt::AdapterPtr MDevAdapter = dev.data()->adapter();

            for (auto tmpdev : MDevAdapter.data()->devices()) {
                qDebug () << Q_FUNC_INFO  << "=======================" << tmpdev.data()->name() << __LINE__;
                if (tmpdev.data()->isConnected() && tmpdev.data()->isPaired()) {
                    if (tmpdev.data()->type() == BluezQt::Device::AudioVideo ||
                        tmpdev.data()->type() == BluezQt::Device::Headphones ||
                        tmpdev.data()->type() == BluezQt::Device::Headset    ) {

                        qDebug () << Q_FUNC_INFO  << "==================disconn=====" << tmpdev.data()->name() << __LINE__;
                        BluezQt::PendingCall *pending = tmpdev.data()->disconnectFromDevice();
                        qDebug () << Q_FUNC_INFO  << "=======================" << __LINE__;
                        connect(pending,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *call){
                            qDebug () << Q_FUNC_INFO  << "=================conn======" << dev.data()->name() << call << __LINE__;

                            if (!call->error()) {
                                BluezQt::PendingCall *call = dev.data()->connectToDevice();
                                GlobalSize::refBlueDevActive(dev->address());
                                connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
                                    qDebug () << Q_FUNC_INFO << callReturn->error() << "   " << callReturn->errorText() << __LINE__;
                                    GlobalSize::unrefBlueDevActive(dev->address());
                                    if (callReturn->error()) {
                                        emit devOperateErrorSignal(dev.data()->address(),
                                                                   callReturn->error(),
                                                                   callReturn->errorText());
                                    }
                                    else
                                    {
                                        //主动连接成功后，发送连接成功信号
                                        addDelay(dev);
                                    }

                                    //主动连接全部完成后，启动扫描
                                    if(GlobalSize::BlueDevActiveSize() == 0)
                                    {
                                         qDebug () << Q_FUNC_INFO  << ::isScan << "===================startDiscovery====" << ::isScan << __LINE__;
                                        if (::isScan) {
                                            BluezQt::PendingCall * start_call = Daemon::_DefualtAdapter.data()->startDiscovery();
                                            connect(start_call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
                                                qDebug () << Q_FUNC_INFO << callReturn->error() << "   " << callReturn->errorText() << __LINE__;
                                            });
                                        }
                                    }

                                });
                            }
                        });
                        //如果有正在连接的音频设备，做完以上流程后，直接退出函数体
                        //因为流程解决定了只有一个音频设备连接，而不需要判断后面是否有音频设备连接；提高代码执行效率
                        return;
                    }
                }
            }
            //当没有任何音频设备连接，最后再连接目标设备
            BluezQt::PendingCall *call = dev.data()->connectToDevice();
            GlobalSize::refBlueDevActive(dev->address());
            connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
                qDebug () << Q_FUNC_INFO << callReturn->error() << "   " << callReturn->errorText() << __LINE__;
                GlobalSize::unrefBlueDevActive(dev->address());
                if (callReturn->error()) {
                    emit devOperateErrorSignal(dev.data()->address(),
                                               callReturn->error(),
                                               callReturn->errorText());
                }
                else
                {
                    //主动连接成功后，发送连接成功信号
                    addDelay(dev);
                }

                //不存在主动连接
                if(GlobalSize::BlueDevActiveSize() == 0)
                {
                    qDebug () << Q_FUNC_INFO  << ::isScan << "===================startDiscovery====" << ::isScan << __LINE__;
                    if (::isScan) {
                        if(!Daemon::_DefualtAdapter.data()->isDiscovering())
                            Daemon::_DefualtAdapter.data()->startDiscovery();
                    }
                }

            });
        }
        //Dt-test phone只连接媒体音频
        else if(dev.data()->type() == BluezQt::Device::Phone)
        {
            qDebug () << Q_FUNC_INFO <<
                         dev.data()->name() <<
                         dev.data()->address() <<
                         dev.data()->type() << __LINE__;

            if (Environment::HUAWEI == envPC)
            {
                //华为
                qDebug () << Q_FUNC_INFO << "Environment::HUAWEI??"<< envPC;
                return;
            }
            //背景：PC连接手机后，PC端播放音频无法从手机端输出
            //手机移动设备只连接媒体音频，可通过手机端发送音频至PC播放.将PC设备作为一个输出设备。
            //UUID: Audio Source              (0000110a-0000-1000-8000-00805f9b34fb)
            BluezQt::PendingCall *call = dev.data()->connectProfile("0000110a-0000-1000-8000-00805f9b34fb");
            GlobalSize::refBlueDevActive(dev->address());
            connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
                qDebug () << Q_FUNC_INFO << callReturn->error() << "   " << callReturn->errorText() << __LINE__;
                GlobalSize::unrefBlueDevActive(dev->address());
                if (callReturn->error()) {
                    emit devOperateErrorSignal(dev.data()->address(),
                                               callReturn->error(),
                                               callReturn->errorText());
                }
                else
                {
                    //主动连接成功后，发送连接成功信号
                    addDelay(dev);
                }

                //不存在主动连接
                if(GlobalSize::BlueDevActiveSize() == 0)
                {
                    qDebug () << Q_FUNC_INFO  << ::isScan << "===================startDiscovery====" << ::isScan << __LINE__;
                    if (::isScan) {
                        if(!Daemon::_DefualtAdapter.data()->isDiscovering())
                            Daemon::_DefualtAdapter.data()->startDiscovery();
                    }
                }
            });
        }
        else {

            qDebug () << Q_FUNC_INFO << dev.data()->type() << __LINE__;

            BluezQt::PendingCall *call = dev.data()->connectToDevice();
            GlobalSize::refBlueDevActive(dev->address());
            connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
                qDebug () << Q_FUNC_INFO << callReturn->error() << "   " << callReturn->errorText() << __LINE__;
                GlobalSize::unrefBlueDevActive(dev->address());
                if (callReturn->error()) {
                    emit devOperateErrorSignal(dev.data()->address(),
                                               callReturn->error(),
                                               callReturn->errorText());
                }
                else
                {
                    //主动连接成功后，发送连接成功信号
                    addDelay(dev);
                }

                if(GlobalSize::BlueDevActiveSize() == 0)
                {
                     qDebug () << Q_FUNC_INFO  << ::isScan << "===================startDiscovery====" << ::isScan << __LINE__;
                    if (::isScan) {
                        if (!Daemon::_DefualtAdapter.data()->isDiscovering())
                        {
                            BluezQt::PendingCall * start_call =  Daemon::_DefualtAdapter.data()->startDiscovery();
                            connect(start_call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
                                qDebug () << Q_FUNC_INFO << callReturn->error() << "   " << callReturn->errorText() << __LINE__;
                            });
                        }
                    }
                }
            });
        }
    }

    qDebug () << Q_FUNC_INFO  << "END" << __LINE__;
}

bool SessionDbusRegister::getBluetoothBlock()
{
    if (Daemon::_BluezManager)
        return Daemon::_BluezManager->isBluetoothBlocked();
}

QString SessionDbusRegister::getDefaultAdapterAddress()
{
    if (!Daemon::_DefualtAdapter.isNull())
        return Daemon::_DefualtAdapter.data()->address();
    else
        return "";
}

bool SessionDbusRegister::getDefaultAdapterPower()
{
    if (!Daemon::_DefualtAdapter.isNull())
        return Daemon::_DefualtAdapter.data()->isPowered();
    else
        return false;
}

void SessionDbusRegister::setDefaultAdapterPower(bool v)
{
    if (!Daemon::_DefualtAdapter.isNull()) {
        if (Daemon::_BluezManager && Daemon::_BluezManager->isBluetoothBlocked())
            Daemon::_BluezManager->setBluetoothBlocked(false);

        qDebug() << Q_FUNC_INFO << "setDefaultAdapterPower" << v << __LINE__;
        Daemon::_DefualtAdapter.data()->setPowered(v)->errorText();
    }
}

bool SessionDbusRegister::getDefaultAdapterScanStatus()
{
    if (!Daemon::_DefualtAdapter.isNull()) {
        return Daemon::_DefualtAdapter.data()->isDiscovering();
    }
    return false;
}

bool SessionDbusRegister::getDefaultAdapterDiscoverable()
{
    if (!Daemon::_DefualtAdapter.isNull()) {
        return Daemon::_DefualtAdapter.data()->isDiscoverable();
    }
    return false;
}

QString SessionDbusRegister::getAdapterNameByAddr(QString addr)
{
    if (addr.isEmpty() || addr.isNull() || !Daemon::_BluezManager)
        return "";

    BluezQt::AdapterPtr adapter = Daemon::_BluezManager->adapterForAddress(addr);
    if (adapter.isNull())
        return "";

    return adapter.data()->name();
}

void SessionDbusRegister::setDefaultAdapterScanOn(bool scanon)
{
    qDebug () << Q_FUNC_INFO  << scanon << __LINE__;
    if (!Daemon::_DefualtAdapter.isNull()) {
//        BluezQt::PendingCall *call = nullptr;
        ::isScan = scanon;
        if (GlobalSize::BlueDevActiveSize() == 0)
        {
            if (scanon) {
                if (!Daemon::_DefualtAdapter.data()->isDiscovering())
                    Daemon::_DefualtAdapter.data()->startDiscovery();
            } else {
                if (Daemon::_DefualtAdapter.data()->isDiscovering())
                    Daemon::_DefualtAdapter.data()->stopDiscovery();
            }
        }
    }
}

void SessionDbusRegister::setDefaultAdapter(QString addr)
{
    if (addr.isEmpty() || addr.isNull() || !Daemon::_BluezManager)
        return;

    BluezQt::AdapterPtr adapter = Daemon::_BluezManager->adapterForAddress(addr);
    if (!adapter.isNull()) {
        Daemon::_DefualtAdapter->disconnect();
        Daemon::_DefualtAdapter = adapter;
        this->connectSignal();
        emit this->defaultAdapterChanged(addr);
    }
}

void SessionDbusRegister::setDefaultAdapterDiscoverable(bool value)
{
    if (!Daemon::_DefualtAdapter.isNull()) {
        qInfo() << Q_FUNC_INFO << Daemon::_DefualtAdapter.data()->discoverableTimeout();
        if (value) {
            Daemon::_DefualtAdapter.data()->setDiscoverableTimeout(0);
        }

        Daemon::_DefualtAdapter.data()->setDiscoverable(value);
    }
}

void SessionDbusRegister::setBluetoothBlock(bool)
{

}

void SessionDbusRegister::setDefaultAdapterName(QString bluetoothName)
{
    if (Daemon::_DefualtAdapter.isNull())
        return;
    Daemon::_DefualtAdapter.data()->setName(bluetoothName);

}

void SessionDbusRegister::devPair(const QString addr)
{
    if (!isAvailableAddress(addr))
        return;

    BluezQt::DevicePtr devPtr = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
    if (!devPtr)
        return;

    BluezQt::PendingCall *call = devPtr.data()->pair();
    connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
        if (callReturn->error()) {
            emit devOperateErrorSignal(devPtr.data()->address(),
                                       callReturn->error(),
                                       callReturn->errorText());
        }
    });
}

void SessionDbusRegister::devConnect(const QString addr)
{
    qDebug () << Q_FUNC_INFO << __LINE__;
    if (!isAvailableAddress(addr))
        return;

    BluezQt::DevicePtr devPtr = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
    if (!devPtr)
        return;


    if (Daemon::_DefualtAdapter.data()->isDiscovering()) {
        ::isScan = true;
        BluezQt::PendingCall * stop_call = Daemon::_DefualtAdapter.data()->stopDiscovery();
        connect(stop_call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn)
        {
            qDebug() << Q_FUNC_INFO << "stopDiscovery:" << callReturn->error() << callReturn->errorText() << __LINE__;
        });
        stop_call->waitForFinished();
    }

    if (devPtr.data()->isPaired()) {
        AudioDevConnectFunc(devPtr);
    } else {
        BluezQt::PendingCall *call = devPtr.data()->pair();
        GlobalSize::refBlueDevActive(devPtr->address());
        connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
            GlobalSize::unrefBlueDevActive(devPtr->address());
            if (callReturn->error()) {
                emit devOperateErrorSignal(devPtr.data()->address(),
                                           callReturn->error(),
                                           callReturn->errorText());
                //主动连接全部完成后，启动扫描
                if(GlobalSize::BlueDevActiveSize() == 0)
                {
                     qDebug () << Q_FUNC_INFO  << ::isScan << "===================startDiscovery====" << ::isScan << __LINE__;
                    if (::isScan) {
                        BluezQt::PendingCall * start_call = Daemon::_DefualtAdapter.data()->startDiscovery();
                        connect(start_call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
                            qDebug () << Q_FUNC_INFO << callReturn->error() << "   " << callReturn->errorText() << __LINE__;
                        });
                    }
                }
            } else
                AudioDevConnectFunc(devPtr);
        });
    }
}

void SessionDbusRegister::devDisconnect(const QString addr)
{
    if (!isAvailableAddress(addr))
        return;

    BluezQt::DevicePtr devPtr = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
    if (!devPtr)
        return;

    BluezQt::PendingCall *call = devPtr.data()->disconnectFromDevice();
    connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
        if (callReturn->error()) {
            emit devOperateErrorSignal(devPtr.data()->address(),
                                       callReturn->error(),
                                       callReturn->errorText());
        }
    });
}

void SessionDbusRegister::devRemove(const QString addr)
{
    if (!isAvailableAddress(addr))
        return;

    BluezQt::DevicePtr devPtr = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
    if (!devPtr)
        return;

    BluezQt::PendingCall *call = Daemon::_DefualtAdapter.data()->removeDevice(devPtr);
    connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
        if (callReturn->error()) {
            emit devOperateErrorSignal(devPtr.data()->address(),
                                       callReturn->error(),
                                       callReturn->errorText());
        } else {
            Config::removeKey(addr);
        }
    });
}

void SessionDbusRegister::devTrust(const QString addr,const bool trust)
{
    if (!isAvailableAddress(addr))
        return;

    BluezQt::DevicePtr devPtr = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
    if (!devPtr)
        return;

    BluezQt::PendingCall *call = devPtr.data()->setTrusted(trust);
    connect(call,&BluezQt::PendingCall::finished,this,[=](BluezQt::PendingCall *callReturn){
        if (callReturn->error()) {
            emit devOperateErrorSignal(devPtr.data()->address(),
                                       callReturn->error(),
                                       callReturn->errorText());
        }
    });
}

bool SessionDbusRegister::getDevPairStatus(const QString addr)
{
    if (addr.isEmpty() || addr.isNull())
        return false;

    if (!Daemon::_DefualtAdapter.isNull()) {
        BluezQt::DevicePtr dev = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
        if (!dev.isNull())
            return dev.data()->isPaired();
        else
            return false;
    } else {
        return false;
    }
}

bool SessionDbusRegister::getDevConnectStatus(const QString addr)
{
    if (addr.isEmpty() || addr.isNull())
        return false;

    if (!Daemon::_DefualtAdapter.isNull()) {
        BluezQt::DevicePtr dev = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
        if (!dev.isNull())
            return dev.data()->isConnected();
        else
            return false;
    } else {
        return false;
    }
}

QString SessionDbusRegister::getDevType(const QString addr)
{
    if (addr.isEmpty() || addr.isNull())
        return "";

    if (!Daemon::_DefualtAdapter.isNull()) {
        BluezQt::DevicePtr dev = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
        if (!dev.isNull())
            return BluezQt::Device::typeToString(dev.data()->type());
        else
            return "";
    } else {
        return "";
    }
}

void SessionDbusRegister::clearAllUnPairedDevicelist()
{
    if (!Daemon::_DefualtAdapter.isNull()) {
        for (auto dev : Daemon::_DefualtAdapter.data()->devices()) {
            if (!dev.data()->isPaired()) {
                Daemon::_DefualtAdapter.data()->removeDevice(dev);
            }
        }
    }
}

qint64 SessionDbusRegister::getDevBattery(const QString addr)
{
    if (addr.isEmpty() || addr.isNull())
        return -1;

#ifdef BATTERY
    if (!Daemon::_DefualtAdapter.isNull()) {
        BluezQt::DevicePtr dev = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
        if (!dev.isNull()) {
            if (dev.data()->battery().isNull())
                return -1;
            return dev.data()->battery().data()->percentage();
        }
    }
#endif
    return -1;
}

QString SessionDbusRegister::getDevName(const QString addr)
{
    if (addr.isEmpty() || addr.isNull() || Daemon::_DefualtAdapter.isNull())
        return "";

    BluezQt::DevicePtr devpter = Daemon::_DefualtAdapter.data()->deviceForAddress(addr);
    if (devpter.isNull())
        return "";

    return devpter.data()->name();
}

QStringList SessionDbusRegister::getDefaultAdapterPairedDevAddress()
{
    QStringList pairDev;
    pairDev.clear();
    if (!Daemon::_DefualtAdapter.isNull()) {
        for (auto item : Daemon::_DefualtAdapter.data()->devices()) {
            if(item.data()->isPaired()) {
                pairDev.append(item.data()->address());
            }
        }
    }
    return pairDev;
}

QStringList SessionDbusRegister::getDefaultAdapterTrustedDevAddress()
{
    QStringList TrusteDev;
    TrusteDev.clear();
    if (!Daemon::_DefualtAdapter.isNull()) {
        for (auto item : Daemon::_DefualtAdapter.data()->devices()) {
            if(item.data()->isTrusted()) {
                TrusteDev.append(item.data()->address());
            }
        }
    }
    return TrusteDev;
}

QStringList SessionDbusRegister::getDefaultAdapterCacheDevAddress()
{
    QStringList CacheDevList;
    CacheDevList.clear();
    if (!Daemon::_DefualtAdapter.isNull()) {
        for (auto item : Daemon::_DefualtAdapter.data()->devices()) {
            CacheDevList.append(item.data()->address());
        }
    }
    return CacheDevList;
}

QStringList SessionDbusRegister::getAdapterDevAddressList()
{
    QStringList AdapterList;
    AdapterList.clear();
    if (Daemon::_BluezManager) {
        for (auto item : Daemon::_BluezManager->adapters()) {
            AdapterList.append(item.data()->address());
        }
    }
    return AdapterList;
}

void SessionDbusRegister::sendFiles(QString addr,QString filename)
{
    if (filename.isEmpty() || filename.isNull())
        return;

    emit sendFile(addr,filename);
}

void SessionDbusRegister::exit()
{
    qApp->exit(0);
}

void SessionDbusRegister::sendNotifyMessage(QString message)
{
    if (message.isEmpty() || message.isNull())
        return;

    Config::SendNotifyMessage(message);
}

void SessionDbusRegister::openBluetoothSettings()
{
    Config::OpenBluetoothSettings();
}

void SessionDbusRegister::connectSignal()
{
    qDebug() << Q_FUNC_INFO ;
    if (nullptr == Daemon::_BluezManager)
    {
        qDebug() << Q_FUNC_INFO << "btManaher is null!" ;
        return;
    }

    if (nullptr == Daemon::_DefualtAdapter)
    {
        qDebug() << Q_FUNC_INFO << "_DefualtAdapter is null!" ;
        return;
    }

    Daemon::_BluezManager->disconnect(this);
    Daemon::_DefualtAdapter->disconnect();

    connect(Daemon::_BluezManager,&BluezQt::Manager::adapterAdded,this,[=](BluezQt::AdapterPtr adapter){
        qDebug() << Q_FUNC_INFO << "adapterAdded" << adapter->address() << adapter->isPowered()<< __LINE__;
        emit adapterAddSignal(adapter.data()->address());

        //修改
        //为默认适配器赋值，daemon未提供接口
        //使用临时方案，只存在一个蓝牙适配器，则直接赋值
        //如存在多个蓝牙适配器情况下，需要进行修改
        //从无到有,以第一个适配器作为默认适配器
        if(Daemon::_BluezManager->adapters().size() == 1)
        {
            Daemon::_DefualtAdapter = adapter;
            //移除设备后，信号消失，需要重新绑定信号
            this->connectSignal();
            if(-1 != static_bluezreboot_power)
            {
                qDebug() << Q_FUNC_INFO << "setPowered" << bool(static_bluezreboot_power);
                qDebug() << Q_FUNC_INFO << "name" << Daemon::_DefualtAdapter->name() << __LINE__;
                Daemon::_DefualtAdapter->setPowered(bool(static_bluezreboot_power));
                static_bluezreboot_power = -1;
            }

        }
        else
        {
            //不做任何处理
        }

    });

    connect(Daemon::_BluezManager,&BluezQt::Manager::adapterRemoved,this,[=](BluezQt::AdapterPtr adapter){
        qDebug() << Q_FUNC_INFO << "adapterRemoved" << __LINE__;
        emit adapterRemoveSignal(adapter.data()->address());
    });

    connect(Daemon::_DefualtAdapter.data(),&BluezQt::Adapter::adapterRemoved,this,[=](BluezQt::AdapterPtr adapter){
        qDebug() << Q_FUNC_INFO << "adapterRemoved" << __LINE__;
        emit adapterRemoveSignal(adapter.data()->address());

        //bluez重启，可能出现蓝牙从开启变成关闭
        qint64 tmp = QDateTime::currentMSecsSinceEpoch() - ::powerChangeTime;
        if(tmp <= 1000){
            qDebug() << Q_FUNC_INFO << "time <= 1000" << __LINE__;
            static_bluezreboot_power = 1;
        }
        else{
            if(adapter->isPowered()){
                qDebug() << Q_FUNC_INFO << "adapter->isPowered() true" << __LINE__;
                static_bluezreboot_power = 1;
            }
            else{
                qDebug() << Q_FUNC_INFO << "adapter->isPowered() false" << __LINE__;
                static_bluezreboot_power = 0;
            }

        }
        qDebug() << Q_FUNC_INFO << "static_bluezreboot_power" << static_bluezreboot_power << __LINE__;
    });

    connect(Daemon::_DefualtAdapter.data(),&BluezQt::Adapter::nameChanged,this,[=](const QString &name)
    {
        qDebug() << Q_FUNC_INFO << "nameChanged" << name << __LINE__;
        emit defaultAdapterNameChanged(name);
    });

    connect(Daemon::_DefualtAdapter.data(),&BluezQt::Adapter::deviceRemoved,this,[=](BluezQt::DevicePtr devPtr){
        qDebug() << Q_FUNC_INFO << "deviceRemoved" << devPtr->address()<<__LINE__;
        emit devRemoveSignal(devPtr.data()->address());

        if (devPtr.data()->isPaired())
            Config::removeKey(devPtr.data()->address());
    });

    connect(Daemon::_DefualtAdapter.data(),&BluezQt::Adapter::poweredChanged,this,[=](bool powered){
        qDebug() << Q_FUNC_INFO << "poweredChanged" << powered <<__LINE__;

        ::powerChangeTime = QDateTime::currentMSecsSinceEpoch();

        if (powered)
            Daemon::reconnectFunc();

        if (::resetFlag || ::rebootFlag || ::sleepFlag) {
            QTimer::singleShot(100,this,[=]{
                Daemon::_DefualtAdapter.data()->setPowered(false);
                qDebug() << Q_FUNC_INFO << __LINE__;
            });
            qDebug() << Q_FUNC_INFO << __LINE__<< ::resetFlag << ::rebootFlag << ::sleepFlag ;
            ::resetFlag  = false;
            ::rebootFlag = false;
            ::sleepFlag  = false;
        }

        Config::gsetting->set(GSETTING_STR_BLUETOOTH_POWER_SWTICH,QVariant(powered));

        emit defaultAdapterPowerChanged(powered);
    });

    connect(Daemon::_DefualtAdapter.data(),&BluezQt::Adapter::discoveringChanged,this,[=](bool status){
        qDebug() << Q_FUNC_INFO << "discoveringChanged" << status << __LINE__;
        emit defaultScanStatusChanged(status);
    });

    connect(Daemon::_DefualtAdapter.data(),&BluezQt::Adapter::discoverableChanged,this,[=](bool discoverable){
        qDebug() << Q_FUNC_INFO << "discoverableChanged" << discoverable << __LINE__;

        //这里是否需要增加intel机型判断，intel默认可被发状态为开启
        Config::gsetting->set(GSETTING_STR_BLUETOOTH_DISCOVERABLE_SWTICH,QVariant(discoverable));

        emit defaultDiscoverableChanged(discoverable);
    });

    connect(Daemon::_DefualtAdapter.data(),&BluezQt::Adapter::deviceAdded,this,[=](BluezQt::DevicePtr devPtr){
//        qInfo() << Q_FUNC_INFO << "deviceAdded:" << devPtr->address()  <<  ";line = "<< __LINE__ ;

        //设备类型正常，才发送信号
        if(devPtr->type() != BluezQt::Device::Type::Uncategorized)
        {
                emit deviceScanResult(devPtr.data()->address(),devPtr.data()->name(),
                                      BluezQt::Device::typeToString(devPtr.data()->type()));
        	//添加mac前3位和名称一致的判断，过滤掉扫描列表中mac地址会变化的设备
        	clearDeviceScanList(devPtr);

        }


        //清空信号,防止重复连接
        devPtr->disconnect();

        connect(devPtr.data(),&BluezQt::Device::connectedChanged,this,[=](bool v){
            qDebug() << Q_FUNC_INFO << "connectedChanged" << devPtr.data()->address() <<
                        v << devPtr.data()->isPaired() << devPtr->isConnected()<< devPtr.data()->type();


            //发送信号
            //主动连接，不发送信号
            //非主动连接情况下
            if(!GlobalSize::IsBlueDevActive(devPtr->address()))
            {
                //条件: 蓝牙适配器：配对、连接， 且蓝牙设备主动连接
                if(devPtr->isPaired() && v)
                {
                    addDelay(devPtr);
                }
                //未配对，且为连接状态， 不发送
                else if(!devPtr->isPaired() && v)
                {
                    ;
                }
                else
                {
                    if(existDelay(devPtr->address()))
                    {
                        qDebug() << Q_FUNC_INFO << __LINE__ << "existDelay " << devPtr->address();
                    }
                    else
                    {
                        qDebug() << Q_FUNC_INFO<< __LINE__ << "emit " << devPtr->address() << v;
                        emit devConnectStatusSignal(devPtr->address(),v);

                        if (v && devPtr->isPaired()) {
                            Config::writeKeyFile(devPtr->address(),devPtr->name(),devPtr->type());
                        }
                    }
                }
            }
        });

        connect(devPtr.data(),&BluezQt::Device::pairedChanged,this,[=](bool v){
            qDebug() << Q_FUNC_INFO << "pairedChanged" << devPtr.data()->address() << v;
            emit devPairSignal(devPtr.data()->address(),v);

            //匹配成功 且 非主动连接情况下 且 蓝牙设备状态 连接
            if(v && !GlobalSize::IsBlueDevActive(devPtr->address()) && devPtr->isConnected())
            {
                addDelay(devPtr);
            }
        });

        connect(devPtr.data(),&BluezQt::Device::rssiChanged,this,[=](bool v){
            //qDebug() << Q_FUNC_INFO << "rssiChanged" << devPtr.data()->address() << "  val = " <<
            ;//       devPtr->rssi() <<  ";line = "<< __LINE__ ;
        });

        connect(devPtr.data(),&BluezQt::Device::typeChanged,this,[=](BluezQt::Device::Type type){
            qDebug() << Q_FUNC_INFO << "typeChanged" << devPtr.data()->address();

            //设备类型正常，才发送信号
            if(devPtr->type() != BluezQt::Device::Type::Uncategorized)
            {
                emit deviceScanResult(devPtr->address(),devPtr->name(),
                                          BluezQt::Device::typeToString(devPtr->type()));

                QTimer::singleShot(500, this, [=]
                {
                    emit devTypeChangedSignal(devPtr->address(),BluezQt::Device::typeToString(devPtr->type()));
                });
            }
            //无蓝牙设备类型，删除设备
            else
            {
                emit devRemoveSignal(devPtr->address());
            }

        });

        connect(devPtr.data(),&BluezQt::Device::nameChanged,this,[=](const QString &name){
            qDebug() << Q_FUNC_INFO << "nameChanged" << devPtr.data()->address();
            emit devNameChangedSignal(devPtr.data()->address(),name);
        });

#ifdef BATTERY
        connect(devPtr.data(),&BluezQt::Device::batteryChanged,this,[=](BluezQt::BatteryPtr battery){
            if(nullptr != battery){
                qDebug() << Q_FUNC_INFO << "batteryChanged" << battery.data()->percentage();
                emit devBatteryChangedSignal(devPtr.data()->address(), QString::number(battery.data()->percentage()));
            }
            else{
                qDebug() << Q_FUNC_INFO << "batteryChanged null " << devPtr.data()->address();
            }

        });
#endif
    });

    foreach (auto dev, Daemon::_DefualtAdapter.data()->devices()) {
        //清空信号，防止重复连接
        dev->disconnect();

        connect(dev.data(),&BluezQt::Device::connectedChanged,this,[=](bool v){
            qDebug() << Q_FUNC_INFO  << "connectedChanged" << dev.data()->address() <<
                        v << dev.data()->isPaired() <<  dev->isConnected() << dev->type();

            //主动连接，在连接完成发送连接信号
            //非主动连接情况下，发送信号
            if(!GlobalSize::IsBlueDevActive(dev->address()))
            {
                //已匹配设备，且改变为连接状态，延时发送，暂时定义2s
                if(dev->isPaired() && v)
                {
                    addDelay(dev);
                }
                //未匹配，且为连接状态， 不发送
                else if(!dev->isPaired() && v)
                {
                    ;
                }
                else
                {
                    if(existDelay(dev->address()))
                    {
                        qDebug() << Q_FUNC_INFO << __LINE__ << "existDelay " << dev->address();
                    }
                    else
                    {
                        qDebug() << Q_FUNC_INFO<< __LINE__ << "emit " << dev->address() << v;
                        emit devConnectStatusSignal(dev.data()->address(),v);

                        if (v && dev.data()->isPaired()) {
                            Config::writeKeyFile(dev.data()->address(),dev.data()->name(),dev.data()->type());
                        }
                    }
                }
            }
        });

        connect(dev.data(),&BluezQt::Device::pairedChanged,this,[=](bool v){
            qDebug() << Q_FUNC_INFO << "pairedChanged" << dev.data()->address();
            emit devPairSignal(dev.data()->address(),v);

            //匹配成功 且 非主动连接情况下 且 蓝牙设备状态 连接
            if(v && !GlobalSize::IsBlueDevActive(dev->address()) && dev->isConnected())
            {
                addDelay(dev);
            }
        });

        connect(dev.data(),&BluezQt::Device::rssiChanged,this,[=](bool v){
            //qDebug() << Q_FUNC_INFO << "rssiChanged" << dev.data()->address() << "  val = " <<
            ;//            dev->rssi() <<  ";line = "<< __LINE__ ;
        });

        connect(dev.data(),&BluezQt::Device::typeChanged,this,[=](BluezQt::Device::Type type){
            qDebug() << Q_FUNC_INFO << "typeChanged" << dev.data()->address();
            //设备类型正常，才发送信号
            if(dev->type() != BluezQt::Device::Type::Uncategorized)
            {
                emit deviceScanResult(dev->address(),dev->name(),
                                          BluezQt::Device::typeToString(dev->type()));

                QTimer::singleShot(500, this, [=]
                {
                    emit devTypeChangedSignal(dev->address(),BluezQt::Device::typeToString(dev->type()));
                });

            }
            //无蓝牙设备类型，删除设备
            else
            {
                emit devRemoveSignal(dev->address());
            }
        });

        connect(dev.data(),&BluezQt::Device::nameChanged,this,[=](const QString &name){
            qDebug() << Q_FUNC_INFO << "nameChanged" << dev.data()->address();
            emit devNameChangedSignal(dev.data()->address(),name);
        });
#ifdef BATTERY
        connect(dev.data(),&BluezQt::Device::batteryChanged,this,[=](BluezQt::BatteryPtr battery){
            if(nullptr != battery){
                qDebug() << Q_FUNC_INFO << "batteryChanged" << battery.data()->percentage();
                emit devBatteryChangedSignal(dev.data()->address(), QString::number(battery.data()->percentage()));
            }
            else{
                qDebug() << Q_FUNC_INFO << "batteryChanged null " << dev.data()->address();
            }
            });
#endif
    }
}

bool SessionDbusRegister::deviceMacAddressContrast(QString newMacAddress , QString oldMacAddress)
{
    //判断mac前三位是否一致
    QString newStr = newMacAddress.left(8);
    QString oldStr = oldMacAddress.left(8);
    if (newStr == oldStr)
        return true;
    else
        return false;
}

void SessionDbusRegister::clearDeviceScanList(BluezQt::DevicePtr devPtr)
{

    if (nullptr == devPtr)
    {
        return;
    }

    foreach (auto dev, Daemon::_DefualtAdapter.data()->devices())
    {
        if( nullptr != dev &&
            !dev.data()->isPaired() &&
            !dev.data()->isConnected() &&
            (devPtr.data()->name() == dev.data()->name()) &&
            (devPtr.data()->address() != dev.data()->address()) &&
             deviceMacAddressContrast(devPtr.data()->address(),dev.data()->address()))
        {
            qDebug() << Q_FUNC_INFO  << devPtr.data()->name() << devPtr.data()->name() ;
            qDebug() << Q_FUNC_INFO  << dev.data()->name() << dev.data()->name() ;

            Daemon::_DefualtAdapter.data()->removeDevice(dev);
            break;
        }
    }
}

void SessionDbusRegister::sendReplyFileReceivingSignal(bool v)
{
    qDebug() << "SessionDbusRegister::sendReplyFileReceivingSignal";
    emit replyFileReceivingSignal(v);
}

void SessionDbusRegister::sendCloseSession()
{
    qDebug() << "SessionDbusRegister::sendCloseSession";
    emit closeSessionSignal();
}


void SessionDbusRegister::sendReplyRequestConfirmation(bool v)
{
    qDebug() << "SessionDbusRegister::sendReplyRequestConfirmation";
    emit replyRequestConfirmation(v);
}

void SessionDbusRegister::setSendTransferFileMesg(QStringList files)
{
    qDebug() << "SessionDbusRegister::setSendTransferFileMesg";
    emit sendTransferFilesMesg(files);
}

void SessionDbusRegister::setSendTransferDeviceMesg(QString address)
{
    qDebug() << "SessionDbusRegister::setSendTransferDeviceMesg";
    emit sendTransferDeviceMesg(address);
}

void SessionDbusRegister::setclearOldSession() {
    qDebug() << "SessionDbusRegister::clearOldSession";
    emit clearOldSession();
}

void SessionDbusRegister::continueSendFiles(QString filename) {
    emit continueSendFilesSignal(filename);
}

void SessionDbusRegister::cancelFileReceiving() {
    qDebug() << "SessionDbusRegister::cancelFileReceiving";
    emit cancelFileReceivingSignal();
}

void SessionDbusRegister::showTrayWidgetUI() {
    qDebug() << "SessionDbusRegister::showTrayWidgetUI";
    emit showTrayWidgetUISignal();
}
