#ifndef SESSIONDBUSREGISTER_H
#define SESSIONDBUSREGISTER_H

#include "daemon.h"
#include "config.h"


#include <KF5/BluezQt/bluezqt/adapter.h>
#include <KF5/BluezQt/bluezqt/manager.h>
#include <KF5/BluezQt/bluezqt/types.h>
#include <KF5/BluezQt/bluezqt/device.h>

#ifdef BATTERY
#include <KF5/BluezQt/bluezqt/battery.h>
#endif

#include <QObject>
#include <QDBusContext>
#include <QDBusConnection>
#include <QDBusConnectionInterface>
#include <QDBusReply>
#include <QDebug>
#include <QString>
#include <QStringList>



class  DelayStruct : public QObject
{
    Q_OBJECT

public:
    DelayStruct(BluezQt::DevicePtr dev, QObject *parent = nullptr);
    ~DelayStruct();

    void  close(void);  //关闭并销毁函数

protected:

    virtual void timerEvent( QTimerEvent *event);

private:
    int m_TimerID = 0;
    BluezQt::DevicePtr m_dev = nullptr;
};


class SessionDbusRegister : public QObject, protected QDBusContext
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.ukui.bluetooth")
public:
    explicit SessionDbusRegister(QObject *parent = nullptr);
    ~SessionDbusRegister();

private:
    bool InterfaceAlreadyExists();
    void AudioDevConnectFunc(BluezQt::DevicePtr);

    void clearDeviceScanList(BluezQt::DevicePtr);
    bool deviceMacAddressContrast(QString,QString);
public slots:
     // 蓝牙默认适配器信息处理dbus
     bool    getBluetoothBlock();

     QString getDefaultAdapterAddress();

     bool    getDefaultAdapterPower();

     bool    getDefaultAdapterScanStatus();

     bool    getDefaultAdapterDiscoverable();

     QString getAdapterNameByAddr(QString);

     QStringList getDefaultAdapterPairedDevAddress();

     QStringList getDefaultAdapterTrustedDevAddress();

     QStringList getDefaultAdapterCacheDevAddress();

     QStringList getAdapterDevAddressList();

     void    setDefaultAdapterPower(bool);

     void    setDefaultAdapterScanOn(bool);

     void    setDefaultAdapter(QString);

     void    setDefaultAdapterDiscoverable(bool);

     void    setBluetoothBlock(bool);

     void    setDefaultAdapterName(QString);

     //蓝牙设备处理dbus
     void devPair(const QString);

     void devConnect(const QString);

     void devDisconnect(const QString);

     void devRemove(const QString);

     void devTrust(const QString,const bool);

     bool getDevPairStatus(const QString);

     bool getDevConnectStatus(const QString);

     QString getDevName(const QString);

     QString getDevType(const QString);

     void clearAllUnPairedDevicelist();

     void setSendTransferDeviceMesg(QString);

     void setSendTransferFileMesg(QStringList files);

     void setclearOldSession();

     void continueSendFiles(QString filename);

     void cancelFileReceiving();
     
//#ifdef BATTERY
     qint64  getDevBattery(const QString);
//#endif

     //文件传输dbus
     void sendFiles(QString,QString);//test

     void exit();

     //其他dbus
     void sendNotifyMessage(QString);

     void openBluetoothSettings();

     void connectSignal();

     void showTrayWidgetUI();

     //test
     void sendReplyRequestConfirmation(bool);
     void sendReplyFileReceivingSignal(bool v);
     void sendCloseSession();

signals:
     //适配器相关信号
     void defaultAdapterPowerChanged(bool);

     void defaultAdapterChanged(QString);

     void defaultScanStatusChanged(bool);

     void defaultDiscoverableChanged(bool);

     void adapterAddSignal(QString);

     void adapterRemoveSignal(QString);

     void defaultAdapterNameChanged(QString);

     //设备相关信号
     void deviceScanResult(QString,QString,QString);

     void devPairSignal(QString,bool);

     void devTypeChangedSignal(QString,QString);

     void devBatteryChangedSignal(QString, QString);

     void devNameChangedSignal(QString,QString);

     void devConnectStatusSignal(QString,bool);

     void devRemoveSignal(QString);

     void devOperateErrorSignal(QString,int,QString);

     //代理相关信号
     void sendFile(QString,QString);

     void continueSendFilesSignal(QString);

     void displayPasskey(QString,QString);

//     void replyDisplayPasskey(bool);

     void requestConfirmation(QString,QString);

     void replyRequestConfirmation(bool);

     void transferredChanged(quint64);

     void statusChanged(QString);

     void fileReceivingSignal(QString,QString,QString,QString,quint64);

     void replyFileReceivingSignal(bool);

     void closeSessionSignal();

     void pairAgentCanceled();

     void obexAgentCanceled();

     void propertyChanged(quint64);

     void sendTransferDeviceMesg(QString);

     void sendTransferFilesMesg(QStringList);

     void clearOldSession();

     void cancelFileReceivingSignal();

     void showTrayWidgetUISignal();
};

#endif // SESSIONDBUSREGISTER_H
