#include "tmpdevclear.h"
#include "daemon.h"

#include <functional>
#include <algorithm>
#include <QDateTime>
#include <QTimer>
#include <QDebug>
#include <QMutexLocker>
#include <KF5/BluezQt/bluezqt/device.h>

//间隔清理不可见设备时间
const qint64 interval = 12*1000;

TmpDevClear* TmpDevClear::_instant = nullptr;

TmpDevClear *TmpDevClear::getInstant()
{
    if (nullptr == _instant) {
        _instant = new TmpDevClear();
    }

    return _instant;
}

void TmpDevClear::init()
{
    baseTime =  new QTimer(this);
    baseTime->setInterval(interval);
    connect(baseTime,&QTimer::timeout,this,&TmpDevClear::removeConnectionFromMap);
    qDebug() << "TmpDevClear currentThreadId :" << QThread::currentThreadId();


    for (auto dev : Daemon::_DefualtAdapter.data()->devices()) {
        qDebug() << Q_FUNC_INFO <<dev.data()->address() << dev.data()->name();
        if (connections.end() != connections.find(dev.data()->address())) {
            QObject::disconnect(connections[dev.data()->address()]);
        }

        devUpdates[dev.data()->address()] = QDateTime::currentMSecsSinceEpoch();
        connections[dev.data()->address()] = connect(dev.data(),&BluezQt::Device::deviceChanged,this,&TmpDevClear::rssiChanaged);
    }

    connect(Daemon::_DefualtAdapter.data(),&BluezQt::Adapter::deviceAdded,this,[=](BluezQt::DevicePtr devAdd){
        if (connections.end() != connections.find(devAdd.data()->address())) {
            QObject::disconnect(connections[devAdd.data()->address()]);
        }

        devUpdates[devAdd.data()->address()] = QDateTime::currentMSecsSinceEpoch();
        connections[devAdd.data()->address()] = connect(devAdd.data(),&BluezQt::Device::deviceChanged,this,&TmpDevClear::rssiChanaged);
    });

    baseTime->start();
}

TmpDevClear::TmpDevClear(QObject *parent): QObject(parent)
{

}

void TmpDevClear::removeConnectionFromMap()
{
    qDebug() << Q_FUNC_INFO << "start" << devUpdates.size() << connections.size();

    for (QMap<QString,qint64>::iterator iter = devUpdates.begin(); iter != devUpdates.end();) {
        if ((iter.value()+interval) <= QDateTime::currentMSecsSinceEpoch()) {
            disconnect(connections[iter.key()]);
            connections.remove(iter.key());

            BluezQt::DevicePtr devptr = Daemon::_DefualtAdapter.data()->deviceForAddress(iter.key());
            if (!devptr.isNull() && !devptr.data()->isPaired()) {
                Daemon::_DefualtAdapter.data()->removeDevice(devptr);
                iter = devUpdates.erase(iter);
            } else {
                iter++;
            }
        } else {
            iter++;
        }
    }

}

void TmpDevClear::rssiChanaged(BluezQt::DevicePtr dev)
{
    Q_UNUSED(dev);

    devUpdates[dev.data()->address()] = QDateTime::currentMSecsSinceEpoch();
}

TmpDevClear::~TmpDevClear()
{

}


