#include "bluetooth.h"

#include <QDebug>
#include <QTranslator>
#include <QApplication>


static QString executeLinuxCmd(QString strCmd)
{
    QProcess p;
    p.start("bash", QStringList() <<"-c" << strCmd);
    p.waitForFinished();
    QString strResult = p.readAllStandardOutput();
    return strResult.toLower();
}

static bool IsHuawei()
{
    QString str = executeLinuxCmd("cat /proc/cpuinfo | grep Hardware");
    if(str.length() == 0)
    {
        return false;
    }

    if(str.indexOf("huawei") != -1 || str.indexOf("pangu") != -1 ||
            str.indexOf("kirin") != -1)
    {
        return true;
    }
    else
    {
        return false;
    }
    return false;
}

Bluetooth::Bluetooth() : mFirstLoad(true) {

    qDebug() << Q_FUNC_INFO << "++ukccBluetooth========================" << __LINE__;
    global_ishuawei = IsHuawei();
    qDebug() << "Ishuawei: "<< global_ishuawei;

    QTranslator * translator = new QTranslator(this);;
    translator->load("/usr/share/ukui-bluetooth/translations/ukcc-bluetooth_" + QLocale::system().name() + ".qm");
    QApplication::installTranslator(translator);

    QList <quint64> btServiceProcessId;
    bool m_bluetooth_service_process_flag = ukccbluetoothconfig::checkProcessRunning(BluetoothServiceExePath/*"bluetoothService"*/,btServiceProcessId);

    for(quint64 tempId : btServiceProcessId)
    {
        qDebug () << Q_FUNC_INFO << "========================" << tempId;
    }

    if (m_bluetooth_service_process_flag)
    {
        qDebug () << Q_FUNC_INFO << BluetoothServiceExePath << "is Running" ;
    }
    else
    {
        qDebug () << Q_FUNC_INFO << BluetoothServiceExePath << "is not Running" ;

        QList <quint64> btTrayProcessId;
        bool m_ukui_bluetooth_process_flag = ukccbluetoothconfig::checkProcessRunning(BluetoothTrayExePath,btTrayProcessId);
        if (m_ukui_bluetooth_process_flag)
        {
            qDebug () << Q_FUNC_INFO << BluetoothTrayExePath << "is Running" ;
            for (quint64 processId:btTrayProcessId)
            {
                qDebug () << Q_FUNC_INFO << "ProcessId:" << btTrayProcessId << "is Running" ;
                ukccbluetoothconfig::killAppProcess(processId);
            }
        }
        else
        {
            qDebug () << Q_FUNC_INFO << BluetoothTrayExePath << "is not Running" ;
        }
        ukccbluetoothconfig::launchBluetoothServiceStart(BluetoothServiceExePath);
    }



    pluginName = tr("Bluetooth");
    bool intel = QFile::exists("/etc/apt/ota_version");
    if (intel)
        pluginType = NETWORK;
    else
        pluginType = DEVICES;
}

Bluetooth::~Bluetooth() {
    if (!mFirstLoad) {
//        delete pluginWidget;
    }
}

QString Bluetooth::plugini18nName() {
    return pluginName;
}

int Bluetooth::pluginTypes() {
    return pluginType;
}

QWidget *Bluetooth::pluginUi() {

    if (mFirstLoad) {
        mFirstLoad = false;
        pluginWidget = new BlueToothMain;
//        pluginWidget = new BlueToothMainWindow;
    }
    if (!mFirstLoad && (nullptr != pluginWidget))
        pluginWidget->setbluetoothAdapterDiscoveringStatus(true);
    return pluginWidget;
}

const QString Bluetooth::name() const {
    return QStringLiteral("Bluetooth");
}

bool Bluetooth::isShowOnHomePage() const
{
    return true;
}

QIcon Bluetooth::icon() const
{
    return QIcon();
}


bool Bluetooth::isEnable() const
{
    QProcess process;
    process.start("rfkill list");
    process.waitForFinished();
    QByteArray output = process.readAllStandardOutput();
    QString str_output = output;
    bool isDevice = str_output.contains(QString("hci"), Qt::CaseInsensitive);

    return isDevice ;
}

void Bluetooth::plugin_leave()
{
    if (nullptr != pluginWidget)
        pluginWidget->setbluetoothAdapterDiscoveringStatus(false);
}

bool global_ishuawei = false;
