#include "deviceselectitem.h"

DeviceSelectItem::DeviceSelectItem(QString name, QString address, QIcon icon, QWidget *parent) :
    QLabel(parent),
    MName(name),
    MAddress(address),
    MIcon(icon),
    _MStatus(Normal)
{
    this->setProperty("useSystemStyleBlur", true);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setFixedSize(440, 50);
    if(QGSettings::isSchemaInstalled("org.ukui.style")){
        StyleSettings = new QGSettings("org.ukui.style");
        connect(StyleSettings,&QGSettings::changed,this,[=]{
            this->update();
        });
    }

    const QByteArray transparency_id(TRANSPARENCY_SETTINGS);
    if(QGSettings::isSchemaInstalled(transparency_id)){
        transparency_gsettings = new QGSettings(transparency_id);
    }
    if(QGSettings::isSchemaInstalled(transparency_id)){
       tran=transparency_gsettings->get(TRANSPARENCY_KEY).toDouble()*255;
       connect(transparency_gsettings, &QGSettings::changed, this, [=] {
           tran=transparency_gsettings->get(TRANSPARENCY_KEY).toDouble()*255;
           this->update();
       });
    }
    this->setToolTip(MName);
}

DeviceSelectItem::~DeviceSelectItem(){

}

void DeviceSelectItem::setUnselectedState() {
    _MStatus = Normal;
    choosed = false;
    this->update();
}

void DeviceSelectItem::setSelectedState() {
    _MStatus = Choose;
    choosed = true;
    QStringList device;
    device.append(MName);
    device.append(MAddress);
    emit clicked(device);
    this->update();
}

QString DeviceSelectItem::getAddress() {
    return MAddress;
}

QColor DeviceSelectItem::getPainterBrushColor()
{
    QColor color;
    switch (_MStatus) {
    case Status::Normal:
        if(StyleSettings->get("style-name").toString() == "ukui-default"
           || StyleSettings->get("style-name").toString() == "ukui-light")
            color = QColor(255,255,255,0);
        else
            color = QColor(0,0,0,0);
        break;
    case Status::Hover:
        if(StyleSettings->get("style-name").toString() == "ukui-default"
                || StyleSettings->get("style-name").toString() == "ukui-light")
            color = QColor(217,217,217,tran);
        else
            color = QColor(55, 55, 59, tran);
        break;
    case Status::Click:
        if(StyleSettings->get("style-name").toString() == "ukui-default"
                || StyleSettings->get("style-name").toString() == "ukui-light")
            color = QColor(209, 209, 209,tran);
        else
            color = QColor(102, 102, 102, tran);
        break;
    case Status::Choose:
            color = QColor(55, 144, 250,tran);
        break;
    default:
        if(StyleSettings->get("style-name").toString() == "ukui-default"
           || StyleSettings->get("style-name").toString() == "ukui-light")
            color = QColor(255,255,255,0);
        else
            color = QColor(0,0,0,0);
        break;
    }
    return color;
}

const QPixmap DeviceSelectItem::loadSvg(const QPixmap &source, const PixmapColor &cgColor)
{
    QImage img = source.toImage();
    for (int x = 0; x < img.width(); x++) {
        for (int y = 0; y < img.height(); y++) {
            auto color = img.pixelColor(x, y);
            if (color.alpha() > 0) {
                switch (cgColor) {
                case PixmapColor::WHITE:
                    color.setRed(255);
                    color.setGreen(255);
                    color.setBlue(255);
                    img.setPixelColor(x, y, color);
                    break;
                case PixmapColor::BLACK:
                    color.setRed(0);
                    color.setGreen(0);
                    color.setBlue(0);
                    img.setPixelColor(x, y, color);
                    break;
                case PixmapColor::GRAY:
                    color.setRed(152);
                    color.setGreen(163);
                    color.setBlue(164);
                    img.setPixelColor(x, y, color);
                    break;
                case PixmapColor::BLUE:
                    color.setRed(61);
                    color.setGreen(107);
                    color.setBlue(229);
                    img.setPixelColor(x, y, color);
                    break;
                default:
                    return source;
                    break;
                }
            }
        }
    }
    return QPixmap::fromImage(img);
}

void DeviceSelectItem::enterEvent(QEvent *event) {
    Q_UNUSED(event);
    if (choosed)
        _MStatus = Choose;
    else
        _MStatus = Hover;
    this->update();
}

void DeviceSelectItem::leaveEvent(QEvent *event) {
    Q_UNUSED(event);
    if (choosed)
        _MStatus = Choose;
    else
        _MStatus = Normal;
    this->update();
}

void DeviceSelectItem::mousePressEvent(QMouseEvent *ev) {
    Q_UNUSED(ev);
    _MStatus = Click;
    this->update();
}

void DeviceSelectItem::mouseReleaseEvent(QMouseEvent *ev) {
    Q_UNUSED(ev);
    choosed = !choosed;
    if (choosed) {
        _MStatus = Choose;
        QStringList device;
        device.append(MName);
        device.append(MAddress);
        emit clicked(device);
    } else {
        _MStatus = Normal;
        emit unclicked();
    }
    this->update();
}

void DeviceSelectItem::paintEvent(QPaintEvent *) {
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setPen(QColor(Qt::transparent));
    painter.setBrush(getPainterBrushColor());
    painter.drawRoundedRect(this->rect(),6,6,Qt::AbsoluteSize);
    painter.setRenderHint(QPainter::SmoothPixmapTransform);
    if(StyleSettings->get("style-name").toString() == "ukui-default" || StyleSettings->get("style-name").toString() == "ukui-light") {
        painter.setPen(QColor(Qt::black));
        style()->drawItemPixmap(&painter, QRect(8, 13, 24, 24), Qt::AlignCenter, MIcon.pixmap(24,24));
    } else {
        painter.setPen(QColor(Qt::white));
        style()->drawItemPixmap(&painter, QRect(8, 13, 24, 24), Qt::AlignCenter, loadSvg(MIcon.pixmap(24,24), PixmapColor::WHITE));
    }
    QString nName;
    nName = MName;
    int fontSize = StyleSettings->get("system-font-size").toInt();
    if (fontSize > 15)
        fontSize = 15;
    if (fontSize < 10)
        fontSize = 10;
    float shownum = 0;
    for (int i = 0; i < MName.size(); i++) {
        QChar ch = MName.at(i);
        ushort uNum = ch.unicode();
        if ((uNum >= 0x4E00 && uNum <= 0x9FA5))
            shownum += 1.7;
        else if (ch == 'm' || ch == 'M')
            shownum += 1.5;
        else
            shownum += 1;
        if (shownum + 1.5 >= FontTable[fontSize - 10]) {
            nName.remove(i, MName.size() - i);
            nName.append("...");
            this->setToolTip(MName);
            break;
        }
    }
    painter.drawText(40,13,380,28,Qt::AlignLeft, nName);
}
