#include "deviceseleterwidget.h"

DeviceSeleterWidget::DeviceSeleterWidget(QMap<QString, QStringList> pairdeviceList, QWidget *parent, QString dev):
    QWidget(parent),
    select_dev(dev),
    deviceList(pairdeviceList)
{
    if(QGSettings::isSchemaInstalled("org.ukui.style")){
        GSettings = new QGSettings("org.ukui.style");
        connect(GSettings,&QGSettings::changed,this,&DeviceSeleterWidget::GSettingsChanges);
    }
    if(QGSettings::isSchemaInstalled("org.ukui.bluetooth")){
        settings = new QGSettings("org.ukui.bluetooth");
    }

    const QByteArray transparency_id(TRANSPARENCY_SETTINGS);
    if(QGSettings::isSchemaInstalled(transparency_id)){
        transparency_gsettings = new QGSettings(transparency_id);
    }
    if(QGSettings::isSchemaInstalled(transparency_id)){
       tran=transparency_gsettings->get(TRANSPARENCY_KEY).toDouble()*255;
       connect(transparency_gsettings, &QGSettings::changed, this, [=] {
           tran=transparency_gsettings->get(TRANSPARENCY_KEY).toDouble()*255;
           this->update();
       });
    }
    this->resize(456,356);
    this->setProperty("useSystemStyleBlur", true);
    this->setAutoFillBackground(false);

    QPalette palette;
    if(GSettings->get("style-name").toString() == "ukui-default"  || GSettings->get("style-name").toString() == "ukui-light"){
        palette.setColor(QPalette::Background,QColor(240,240,240,tran));
    }else{
        palette.setColor(QPalette::Background,QColor(0,0,0,tran));
    }
    this->setPalette(palette);

    if (select_dev == "")
        select_dev.clear();

    QVBoxLayout *main_layout = new QVBoxLayout(this);
    main_layout->setSpacing(5);
    main_layout->setContentsMargins(0,0,0,0);

    m_scroll = new QScrollArea(this);
    m_scroll->setWidgetResizable(true);
    m_scroll->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_scroll->setProperty("useSystemStyleBlur", true);
    m_scroll->setAttribute(Qt::WA_TranslucentBackground);
    m_scroll->setProperty("drawScrollBarGroove",false);
    m_scroll->verticalScrollBar()->setProperty("drawScrollBarGroove",false);
//    m_scroll->setFixedSize(376,240);
    main_layout->addWidget(m_scroll);

    dev_widget = new QWidget();
    m_scroll->setWidget(dev_widget);

    device_list_layout = new QVBoxLayout(dev_widget);
    device_list_layout->setSpacing(5);
    device_list_layout->setContentsMargins(8,8,8,8);
    dev_widget->setLayout(device_list_layout);

    InitUI();
}

DeviceSeleterWidget::~DeviceSeleterWidget()
{

}

void DeviceSeleterWidget::InitUI()
{
    bool exit_device = false;
    for(int i=0; i < deviceList.values().size(); i++){
//        qDebug() << Q_FUNC_INFO << device_list.at(i)->type() << device_list.at(i)->name();
        BluezQt::Device::Type type = BluezQt::Device::stringToType(deviceList.values().at(i).at(1));
        if((type == BluezQt::Device::Phone)||(type == BluezQt::Device::Computer)){
                QIcon icon;
                switch (type){
                case BluezQt::Device::Type::Phone:
                    icon = QIcon::fromTheme("phone-apple-iphone-symbolic");
                    break;
                case BluezQt::Device::Type::Computer:
                    icon = QIcon::fromTheme("video-display-symbolic");
                    break;
                case BluezQt::Device::Type::Uncategorized:
                default:
                    icon = QIcon::fromTheme("bluetooth-symbolic");
                    break;
                }

                DeviceSelectItem *item = new DeviceSelectItem(deviceList.values().at(i).at(0), deviceList.key(deviceList.values().at(i)),icon, dev_widget);
                connect(item,&DeviceSelectItem::clicked,this,&DeviceSeleterWidget::itemToolbuttonClicked);
                connect(item, &DeviceSelectItem::unclicked, this, [=] {
                    emit unsign_select();
                    select_name.clear();
                    select_dev.clear();
                });
                if (select_dev == deviceList.key(deviceList.values().at(i))) {
                    item->setSelectedState();
                    exit_device = true;
                }
                device_list_layout->addWidget(item, Qt::AlignTop |Qt::AlignLeft);
                toolbutton_list.append(item);
        }
    }
    if (!exit_device) {
        select_dev.clear();
        select_name.clear();
    }

    if(toolbutton_list.size() == 0){
        QLabel *tip_label = new QLabel(this);
        tip_label->setFixedSize(440, 50);
        tip_label->setAlignment(Qt::AlignCenter);
        tip_label->setText(tr("No device currently available \n Please go to pair the device"));
        device_list_layout->addWidget(tip_label,Qt::AlignCenter);
    }else{
        device_list_layout->addStretch();
    }
}

QString DeviceSeleterWidget::get_seleter_device()
{
    qDebug() << Q_FUNC_INFO;
    return select_dev;
}

QString DeviceSeleterWidget::get_seleter_dev_name()
{
    return select_name;
}

void DeviceSeleterWidget::itemToolbuttonClicked(QStringList device)
{
    for (int i = 0; i < toolbutton_list.size(); i++) {
        if (select_dev.isNull())
            break;
        if (toolbutton_list.at(i)->getAddress() == device.at(1))
            continue;
        toolbutton_list.at(i)->setUnselectedState();
    }
    select_name = device.at(0);
    select_dev = device.at(1);

    emit sign_select();
}

void DeviceSeleterWidget::GSettingsChanges(const QString &key)
{
    QPalette palette;
    qDebug() << Q_FUNC_INFO << key;
    if(key == "styleName"){
        if(GSettings->get("style-name").toString() == "ukui-default" || GSettings->get("style-name").toString() == "ukui-light"){
            palette.setColor(QPalette::Background,QColor(240,240,240,tran));
        }else{
            palette.setColor(QPalette::Background,QColor(0,0,0,tran));
        }
    }
    this->setPalette(palette);
}
